;(function(window, document, undefined) {
	'use strict';

	var createEvents = (function(actions) {
		return function(arr) {
			(function(methods, selector, route, options) {
				methods.forEach(function(v) {
					var elements = document.querySelectorAll(selector + ' .' + v);
					Array.prototype.forEach.call(elements, function(w) {
						w.addEventListener(
							options && options['listener'] ? options['listener'] : 'click',
							actions[v](selector, route && (route + v), options),
							false);
					});
				});
			}).apply(null, arr);
		}
	})((function() {
		function icon(className) {
			return [
				['li'],
				['a', { href: '#' }],
				['i', { classes: ['fa', 'fa-fw', className] }]
			].map(function(w) {
				return createElement.apply(null, w);
			});
		}

		function action(icon, action, attributes) {
			Object.keys(attributes || []).forEach(function(v) {
				if (Object.prototype.hasOwnProperty.call(attributes, v)) {
					icon[icon.length - 1].setAttribute(v, attributes[v]);
				}
			});

			return [icon, action];
		}

		function formInput(element, form, options, icons, value) {
			var input = createElement('input', {
					classes: ['form-control', 'form-minicontrol', 'flex-fill'],
					type: 'text',
					name: 'name',
					maxLength: options.maxLength,
					value: value || ''
				}),
				toolbox = createElement('ul', {
					classes: ['nav', 'panel_toolbox', 'flex-fixed']
				});

			icons.forEach(function(v) {
				[toolbox]
					.concat(v[0])
					.reduce(function(a, b) {
						a.appendChild(b);
						return b;
					})
					.addEventListener('click', v[1](element, form, input), false);
			});

			return [input, toolbox];
		}

		function form(element, action, options, parent, icons, id, value) {
			var form = createElement('form', {
					classes: ['form-group', 'flex', 'no-bmargin', 'no-vpadding'],
					method: 'post',
					action: action
				}),
				csrf = createElement('input', {
					type: 'hidden',
					name: 'csrf_token',
					value: window.csrfToken
				}),
				parent = createElement('input', {
					type: 'hidden',
					name: 'parent',
					value: parent || ''
				}),
				hidden = id && createElement('input', {
					type: 'hidden',
					name: 'id',
					value: id
				});

			[csrf, parent, hidden].concat(formInput(element, form, options, icons, value))
				.filter(function(v) { return v; })
				.forEach(function(v) {
					form.appendChild(v);
				});

			form.addEventListener('submit', function(e) {
				e && !input.value.trim().length && e.preventDefault() && e.stopPropagation();
			});

			return form;
		}

		function replace(element, form) {
			var next = element.nextSibling;
			var parent = element.parentNode;

			element.style.display = 'none';
			next && parent.insertBefore(form, next) || parent.appendChild(form);
		}

		function getFormInput(form) {
			return form.querySelector('input[type="text"]');
		}

		var actions = {
			save: function(element, form, input) {
				return function(e) {
					e && e.preventDefault() && e.stopPropagation();
					input.value = input.value.trim();
					input.value.length && form.submit();
				};
			},

			close: function(element, form, input) {
				return function(e) {
					e && e.preventDefault() && e.stopPropagation();
					element.style.display = '';
					form.parentNode.removeChild(form);
				};
			},

			remove: function(modal, id) {
				return function(element, form, input) {
					return function(e) {
						e && e.preventDefault() && e.stopPropagation();
						document.querySelector(modal + ' input[type="hidden"][name="id"]').value = id;
					}
				}
			}
		};

		return {
			add: function(selector, route, options) {
				return function(e) {
					e && e.preventDefault() && e.stopPropagation();

					var icons = [
						action(icon('fa-floppy-o'), actions.save),
						action(icon('fa-times'), actions.close)
					];

					var f = form(this, route, options, this.dataset.parentId, icons);
					replace(this, f);
					getFormInput(f).focus();
				};
			},

			edit: function(selector, route, options) {
				var modal = '#' + selector.substr(1) + '-warning-delete-modal';
				var modalAttributes = {
					'data-toggle': 'modal',
					'data-target': modal
				};

				var submit = document.querySelector(modal + ' [data-submit]');
				if (submit.dataset['actionReady'] !== 'true') {
					submit.dataset['actionReady'] = 'true';
					submit.addEventListener('click', function(e) {
						e && e.preventDefault() && e.stopPropagation();
						e.target.closest('form').submit();
					});
				}

				return function(e) {
					e && e.preventDefault() && e.stopPropagation();

					var item = this.closest('.admin-item');
					var id = parseInt(item.dataset.id);
					var value = item.querySelector('.admin-item-value').innerText;

					var icons = [
						action(icon('fa-floppy-o'), actions.save),
						action(icon('fa-trash-o'), actions.remove(modal, id), modalAttributes),
						action(icon('fa-times'), actions.close),
					];

					var f = form(item, route, options, this.dataset.parentId, icons, id, value);
					replace(item, f);
					getFormInput(f).focus();
				};
			},

			category_edit: function(selector, route, options) {
				var hidden_form = document.querySelector('#admin-categories-edit-form');

				var field_id = document.querySelector('#admin-categories-edit-id'),
					field_name = document.querySelector('#admin-categories-edit-name'),
					category_fields = document.querySelector('.admin-category-fields');

				var modal_id = '#admin-categories-warning-delete-modal',
					category_edit = document.querySelector('#admin-categories-edit-submit'),
					category_delete = document.querySelector('#admin-categories-delete');

				if (category_delete.dataset['actionReady'] !== 'true') {
					category_delete.dataset['actionReady'] = true;
					category_delete.dataset['toggle'] = 'modal';
					category_delete.dataset['target'] = modal_id;

					category_edit.addEventListener('click', function(e) {
						e && e.preventDefault() && e.stopPropagation();
						var id = field_id.value;
						hidden_form.querySelector('[name="id"]').value = id;
						var name = field_name.value.trim();
						hidden_form.querySelector('[name="name"]').value = name;

						var new_fields_i = 0, valid_fields = true;
						Array.prototype.every.call(category_fields.querySelectorAll('input'), function(v) {
							var field_id = v.dataset.id || null;
							var category_id = field_id && v.dataset.categoryId;
							var value = v.value.trim();

							if (value && (!field_id || category_id == id)) {
									if (!v.dataset['delete']) {
									var input = createElement('input', {
										type: 'hidden',
										name: field_id ? 'fields-' + field_id : 'new_fields-' + (new_fields_i++),
										value: value
									});

									hidden_form.appendChild(input);
								}
							} else if (field_id && category_id == id) {
								valid_fields = false;
								return false;
							}

							return true;
						});

						if (name && valid_fields) {
							hidden_form.submit();
						}
					});
				}

				return function(e) {
					e && e.preventDefault() && e.stopPropagation();

					var category = e.target.closest('.nav').closest('li').querySelector('.photonics_category'),
						modal = document.querySelector(modal_id);

					var id = parseInt(category.dataset.category),
						name = category.querySelector('.photonics_category_title').innerText;

					Array.prototype.forEach.call(category_fields.querySelectorAll('input'), function(v) {
						var li = v.closest('li');
						if (v.classList.contains('category-field')) {
							if (v.dataset['categoryId'] == id) {
								delete v.dataset['delete'];
								v.value = v.dataset['value'];
								li.style.display = null;
							} else {
								li.style.display = 'none';
							}
						} else {
							li.parentNode.removeChild(li);
						}
					});

					field_id.value = id;
					field_name.value = name;
					
					modal.querySelector('input[type="hidden"][name="id"]').value = id;
				}
			},

			category_field_add: function(selector, route, options) {
				return function(e) {
					e && e.preventDefault() && e.stopPropagation();

					var div = createElement('div', { classes: ['form-group', 'flex', 'no-bmargin', 'no-vpadding'] });
					var icons = [];
					formInput(this, undefined, options, icons)
						.forEach(function(v) {
							div.appendChild(v);
						});

					var li = createElement('li');
					li.appendChild(div);

					var parent = this.closest('li'), list = this.closest('ul');
					(parent.nextSibling && list.insertBefore(li, parent.nextSibling)) || list.appendChild(li);

					getFormInput(div).focus();
				};
			},

			category_field_delete: function(selector, route, options) {
				return function(e) {
					e && e.preventDefault() && e.stopPropagation();
					var field = this.closest('.nav').closest('li').querySelector('.category-field');
					field.dataset['delete'] = 'true';
					field.closest('li').style.display = 'none';
				}
			},

			clear: function(selector, route, options) {
				return function(e) {
					e && e.preventDefault() && e.stopPropagation();
					var parent = this.closest(selector);
					parent.querySelector(options.selector).value = null;
				}
			},

			recover_backup: function(selector, route, options) {
				return function(e) {
					if (e.target.files.length === 1) {
						e.target.closest('form').submit();
					}
				}
			}
		}
	})());

	var adminActions = [
		[['add', 'edit'], '.admin-entry-status-types', 'admin/entry/status/', { maxLength: 64 }],
		[['add', 'edit'], '.admin-entry-documentation-types', 'admin/entry/documentation/', { maxLength: 64 }],
		[['add', 'edit'], '.admin-entry-use-types', 'admin/entry/use/', { maxLength: 64 }],
		[['add', 'edit'], '.admin-entry-fields', 'admin/entry/field/', { maxLength: 64 }],
		[['add', 'edit'], '.admin-user-types', 'admin/user/type/', { maxLength: 64 }],
		[['add', 'category_edit'], '.admin-categories', 'admin/category/', { maxLength: 128 }],
		[['category_field_add', 'category_field_delete'], '.admin-category-fields', null, {maxLength: 64}],
		[['clear'], '.admin-notifications', null, { selector: 'textarea' }],
		[['recover_backup'], '.admin-recover-backup', null, { listener: 'change' }]
	];

	adminActions.forEach(createEvents);

})(window, document);
