#!/usr/bin/env sh

if [[ $EUID -ne 0 ]]; then
  echo "El script solo puede ser ejecutado con permisos de superusuario" 2>&1
  exit 1
fi

PYTHON_MAIN_VERSION=3.5
PYTHON_VERSION=3.5.1

POSTGRESQL_DOTTED_VERSION=9.5
POSTGRESQL_MAIN_VERSION=95
POSTGRESQL_VERSION=9.5-2

NGINX_VERSION=1.10.1

function get_app_tar() {
cat << 'EOF' | openssl base64 -d > /tmp/photonics.tar
EOF
}

localectl set-keymap es

yum -y check-update
yum -y update

/bin/cp /usr/share/zoneinfo/Europe/Madrid /etc/localtime

yum -y install chrony
systemctl unmask chronyd
systemctl enable chronyd
systemctl start chronyd

yum -y install firewalld
systemctl unmask firewalld
systemctl enable firewalld
systemctl start firewalld

yum -y install openssl openssl-devel
yum -y install policycoreutils-python
yum -y install policycoreutils-python-utils
yum -y install unzip
yum -y install gcc
yum -y install gcc-c++
yum -y install vim
yum -y install wget

yum -y install python-devel

PHOTONICS_KEY=$(openssl rand -base64 12)
DATABASE_ADMIN_PASSWORD=$(openssl rand -base64 24)
DATABASE_PASSWORD=$(openssl rand -base64 20)

export PATH=$PATH:/usr/local/bin

cd /tmp

# Python
wget --no-check-certificate https://www.python.org/ftp/python/${PYTHON_VERSION}/Python-${PYTHON_VERSION}.tgz
tar xzvf Python-${PYTHON_VERSION}.tgz
cd Python-${PYTHON_VERSION}
./configure --prefix=/usr/local LDFLAGS="-Wl,-rpath /usr/local/lib" --with-ensurepip=install
make && make altinstall

pip${PYTHON_MAIN_VERSION} install --upgrade pip

easy_install-${PYTHON_MAIN_VERSION} virtualenv
easy_install-${PYTHON_MAIN_VERSION} virtualenvwrapper

# PostgreSQL
rpm -Uvh https://download.postgresql.org/pub/repos/yum/${POSTGRESQL_DOTTED_VERSION}/redhat/rhel-7-x86_64/pgdg-centos${POSTGRESQL_MAIN_VERSION}-${POSTGRESQL_VERSION}.noarch.rpm
yum -y install postgresql${POSTGRESQL_MAIN_VERSION}-server
yum -y install postgresql${POSTGRESQL_MAIN_VERSION}-devel

/usr/pgsql-${POSTGRESQL_DOTTED_VERSION}/bin/postgresql${POSTGRESQL_MAIN_VERSION}-setup initdb

systemctl unmask postgresql-${POSTGRESQL_DOTTED_VERSION}
systemctl start postgresql-${POSTGRESQL_DOTTED_VERSION}
systemctl enable postgresql-${POSTGRESQL_DOTTED_VERSION}

sudo -u postgres createuser photonics
sudo -u postgres createdb photonics

sudo -u postgres psql --command "GRANT ALL PRIVILEGES ON DATABASE photonics TO photonics"

sudo -u postgres psql --command "ALTER USER postgres WITH PASSWORD '${DATABASE_ADMIN_PASSWORD}'"
sudo -u postgres psql --command "ALTER USER photonics WITH PASSWORD '${DATABASE_PASSWORD}'"

awk '$1=="local" && $2=="all" {$4="md5"} $1=="host" && $2=="all" {$5="md5"} 1' /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/pg_hba.conf > /tmp/pg_hba.conf
awk '$1=="#listen_addresses" {$1="listen_addresses";$3="'"'*'"'"} 1' /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/postgresql.conf > /tmp/postgresql.conf
/bin/mv /tmp/pg_hba.conf /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/pg_hba.conf
/bin/mv /tmp/postgresql.conf /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/postgresql.conf
chown postgres:postgres /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/pg_hba.conf
chown postgres:postgres /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/postgresql.conf
chmod 600 /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/pg_hba.conf
chmod 600 /var/lib/pgsql/${POSTGRESQL_DOTTED_VERSION}/data/postgresql.conf

systemctl restart postgresql-${POSTGRESQL_DOTTED_VERSION}

cd /tmp

# Nginx
cat > /etc/yum.repos.d/nginx.repo <<-EOF
	[nginx]
	name=nginx repo
	baseurl=http://nginx.org/packages/centos/7/\$basearch/
	gpgcheck=0
	enabled=1
EOF

yum -y install nginx

adduser \
	--system \
	--no-create-home \
	--shell /sbin/nologin \
	nginx

/bin/mv /etc/nginx/nginx.conf /etc/nginx/nginx.conf.bak

cat > /etc/nginx/nginx.conf <<-EOF
	user nginx;
	worker_processes auto;
	error_log /var/log/nginx/error.log crit;
	pid /run/nginx.pid;

	events {
	    worker_connections 1024;
	}

	http {
	    log_format  main  '\$remote_addr - \$remote_user [\$time_local] "\$request" '
	                      '\$status \$body_bytes_sent "\$http_referer" '
	                      '"\$http_user_agent" "\$http_x_forwarded_for"';

	    access_log  off;

	    sendfile            on;
	    tcp_nopush          on;
	    tcp_nodelay         on;
	    keepalive_timeout   65;
	    types_hash_max_size 8192;
	    client_max_body_size 8192M;

	    include             /etc/nginx/mime.types;
	    default_type        application/octet-stream;

	    include /etc/nginx/conf.d/*.conf;

	    server {
	        listen       80 default_server;
	        listen       [::]:80 default_server;
	        server_name  fotonica.unavarra.es;

	        include /etc/nginx/default.d/*.conf;

	        location / {
	            include uwsgi_params;
	            uwsgi_pass unix:/var/www/photonics/uwsgi.sock;
	            uwsgi_read_timeout 86400;
	        }

	        location /static {
	            root /var/www/photonics/htdocs;
	            gzip on;
	            gzip_min_length 1024;
	            gzip_buffers 4 32;
	        }

            location ~ /(400|403|404|500|502).html {
                root /var/www/photonics/htdocs/templates;
	            gzip on;
	            gzip_min_length 1024;
	            gzip_buffers 4 32;
	        }

	        error_page 400 /400.html;
	        error_page 403 /403.html;
	        error_page 404 /404.html;
	        error_page 500 /500.html;
	        error_page 502 /502.html;
	    }
	}
EOF

systemctl unmask nginx
systemctl start nginx
systemctl enable nginx

# App
mkdir /var/www
mkdir /var/www/photonics
mkdir /var/www/photonics/htdocs
mkdir /var/www/photonics/files
mkdir /var/www/photonics/images
mkdir /var/www/photonics/backups

adduser --no-create-home --home-dir /var/www/photonics --no-user-group --gid nginx --shell /bin/bash photonics
echo 'photonics ALL=(ALL) NOPASSWD: /bin/yum -y update' >> /etc/sudoers
echo 'photonics ALL=(ALL) NOPASSWD: /sbin/reboot' >> /etc/sudoers
sed -i -e 's/Defaults    requiretty/#Defaults    requiretty/g' /etc/sudoers

echo "photonics:"$PHOTONICS_KEY | chpasswd

chown nginx:nginx /var/www

cd /var/www/photonics

virtualenv env
. env/bin/activate

export PATH=$PATH":"$(dirname $(find / -name "pg_config" -print -quit))

pip install uwsgi
pip install psycopg2
pip install flask
pip install flask-sqlalchemy
pip install sqlalchemy-searchable
pip install flask-login
pip install flask-wtf
pip install flask-babelex
pip install orderedset
pip install zipstream

pip install sqlparse

yum -y install libxml2-devel
yum -y install libxslt-devel
pip install lxml

get_app_tar
tar -xf /tmp/photonics.tar -C /

chown photonics:nginx /var/www/photonics
chmod 750 /var/www/photonics

chown photonics:nginx /var/www/photonics/htdocs
chmod 750 /var/www/photonics/htdocs

chown photonics:nginx /var/www/photonics/files
chmod 750 /var/www/photonics/files

chown photonics:nginx /var/www/photonics/images
chmod 750 /var/www/photonics/images

chown photonics:nginx /var/www/photonics/backups
chmod 750 /var/www/photonics/backups

chown photonics:nginx /var/www/photonics/babel.cfg
chmod 644 /var/www/photonics/babel.cfg

chown photonics:nginx /var/www/photonics/messages.pot
chmod 644 /var/www/photonics/messages.pot

chown photonics:nginx /var/www/photonics/uwsgi.ini
chmod 644 /var/www/photonics/uwsgi.ini

chown photonics:nginx /var/www/photonics/update.sh
chmod 750 /var/www/photonics/update.sh

chown photonics:nginx /var/www/photonics/babel
chmod 750 /var/www/photonics/babel

find /var/www/photonics/babel -type d -print0 | xargs -0 chown photonics:nginx
find /var/www/photonics/babel -type d -print0 | xargs -0 chmod 750

find /var/www/photonics/babel -type f -print0 | xargs -0 chown photonics:nginx
find /var/www/photonics/babel -type f -print0 | xargs -0 chmod 640

chown photonics:nginx /var/www/photonics/htdocs
chmod 750 /var/www/photonics/htdocs

find /var/www/photonics/htdocs -type d -print0 | xargs -0 chown photonics:nginx
find /var/www/photonics/htdocs -type d -print0 | xargs -0 chmod 750

find /var/www/photonics/htdocs -type f -print0 | xargs -0 chown photonics:nginx
find /var/www/photonics/htdocs -type f -print0 | xargs -0 chmod 640

sed -i -e "s/{DATABASE_PASSWORD}/"${DATABASE_PASSWORD}"/" /var/www/photonics/htdocs/config.py

cd /var/www/photonics/htdocs/
python app.py createdb

deactivate

# WSGI
cat > /etc/systemd/system/photonics.service <<-EOF
	[Unit]
	Description=Photonics app uWSGI instance
	After=network.target

	[Service]
	User=photonics
	Group=nginx
	WorkingDirectory=/var/www/photonics/
	Environment="PATH=/var/www/photonics/env/bin"
	ExecStart=/var/www/photonics/env/bin/uwsgi --ini /var/www/photonics/uwsgi.ini

	[Install]
	WantedBy=multi-user.target
EOF

systemctl unmask photonics
systemctl start photonics
systemctl enable photonics

# FTP
yum -y install vsftpd
systemctl unmask vsftpd
systemctl start vsftpd
systemctl enable vsftpd

# SELinux
cat > /tmp/nginx.te <<-EOF
	module nginx 1.0;

	require {
	    type init_t;
	    type httpd_t;
	    type httpd_sys_content_t;
	    type var_t;
	    class unix_stream_socket connectto;
	    class sock_file write;
	    class dir read;
	    class file getattr;
	    class file open;
	    class file read;
	    class file write;
	}

	#============= httpd_t ==============
	allow httpd_t init_t:unix_stream_socket connectto;
	allow httpd_t httpd_sys_content_t:sock_file write;
	allow httpd_t var_t:sock_file write;
	allow httpd_t var_t:dir read;
	allow httpd_t var_t:file open;
	allow httpd_t var_t:file getattr;
	allow httpd_t var_t:file read;
	allow httpd_t var_t:file write;
EOF

checkmodule -M -m -o /tmp/nginx.mod /tmp/nginx.te
semodule_package -o /tmp/nginx.pp -m /tmp/nginx.mod
semodule -i /tmp/nginx.pp

# Firewall
firewall-cmd --zone=public --add-service=http --permanent
firewall-cmd --zone=public --add-service=postgresql --permanent
firewall-cmd --zone=public --add-service=ftp --permanent
firewall-cmd --reload

# Cleanup
/bin/rm -Rf /tmp/*

tput setaf 2
echo
echo 'Estas son las contraseñas de acceso a los diferentes servicios del sistema.'
echo 'Guárdelas en un lugar seguro ya que no será posible recuperarlas más tarde.'
echo
echo -e '\tSSH y FTP  - usuario photonics: '${PHOTONICS_KEY}
echo -e '\tPostgreSQL - usuario postgres:  '${DATABASE_ADMIN_PASSWORD}
echo -e '\tPostgreSQL - usuario photonics: '${DATABASE_PASSWORD}
echo
echo 'Es posible modificar la contraseña del usuario photonics desde una conexión'
echo 'SSH con el comando passwd. Sin embargo, las contraseñas de acceso a la base'
echo 'de datos PostgreSQL no deben modificarse manualmente en ningún caso.'
echo
echo 'La dirección IP actual del servidor es '$(ip addr | awk '$1=="inet" && $5!="lo" {print $2}' | sed 's/\/.*//')
echo
echo 'Es necesario reiniciar el servidor para que la aplicación sea operativa.'
echo
tput sgr 0
