from flask_sqlalchemy import SQLAlchemy
from sqlalchemy_searchable import make_searchable

from flask_babelex import gettext
from wtforms.fields.core import UnboundField

import datetime
import json

db = SQLAlchemy()
make_searchable()

class BabelText(object):
	def __init__(self, str):
		self.__str = str

	def __str__(self):
		return gettext(self.__str)

	def __repr__(self):
		return self.__str

	def to_json(self):
		return str(self)

class DatabaseUtils(object):
	def __init__(self):
		import models.app
		import models.account
		import models.inventory

		self.model_order = [
			models.app.Notification,
			models.account.UserType,
			models.account.User,
			models.inventory.Category,
			models.inventory.CategoryField,
			models.inventory.EntryStatusType,
			models.inventory.EntryDocumentationType,
			models.inventory.EntryUseType,
			models.inventory.EntryField,
			models.inventory.Entry,
			models.inventory.EntryFile]

	@staticmethod
	def create_database_schema():
		db.configure_mappers()
		db.create_all()
		db.session.commit()

	@staticmethod
	def drop_database_schema():
		db.drop_all()
		db.clear_mappers()
		db.engine.execute("DROP FUNCTION IF EXISTS entries_name_search_vector_update();")
		db.engine.execute("DROP FUNCTION IF EXISTS entries_description_search_vector_update();")
		db.engine.execute("DROP FUNCTION IF EXISTS entries_comments_search_vector_update();")
		db.engine.execute("DROP FUNCTION IF EXISTS entries_full_search_vector_update();")
		db.engine.execute("DROP FUNCTION IF EXISTS entries_full_search_vector_update();")

		db.session.commit()

	def create_database_backup(self):
		tables = {}
		for model in self.model_order:
			table = []
			for row in model.query.all():
				table.append(self.__model_as_dict(row))

			tables[model.__tablename__] = table

		return json.dumps(tables, default=DatabaseUtils.database_serializer)

	def recover_database_backup(self, obj):
		tables = json.loads(obj)

		for model in self.model_order[::-1]:
			for row in model.query.all():
				db.session.delete(row)

		db.session.commit()

		for model in self.model_order:
			for row in tables[model.__tablename__]:
				row = self.__dict_as_model(model, row)
				db.session.add(row)

			if 'id' in [c.name for c in model.__table__.columns]:
				if len(tables[model.__tablename__]):
					id = max(r['id'] for r in tables[model.__tablename__]) + 1
					db.engine.execute('ALTER SEQUENCE ' + model.__tablename__ + '_id_seq RESTART WITH ' + str(id) + ';')
				else:
					db.engine.execute('ALTER SEQUENCE ' + model.__tablename__ + '_id_seq RESTART WITH 1;')


			db.session.commit()

	def __model_as_dict(self, row):
		return { c.name: getattr(row, c.name) for c in row.__table__.columns }

	def __dict_as_model(self, table, row):
		data = table()
		for c in table.__table__.columns:
			setattr(data, c.name, row[c.name])

		return data

	@staticmethod
	def database_serializer(obj):
		if isinstance(obj, datetime.date):
			serial = obj.isoformat()
			return serial

		raise TypeError("Type not serializable " + str(obj))
		