;(function(window, document, undefined) {
	'use strict';

	(function entryCollapse() {
		var entries = document.getElementsByClassName('photonics_entry');

		Array.prototype.forEach.call(entries, function(entry) {
			var collapseLink = entry.querySelector('.collapse-link'),
				collapseTitle = entry.querySelector('.x_title h2');

			collapseLink && [collapseLink, collapseTitle].forEach(function(v) {
				v && v.addEventListener(
					'click',
					makeCollapser(entry, collapseLink.querySelector('i')),
					false);
			});
		});
	})();

	(function entryImageAlign() {
		var images = document.getElementsByClassName('photonics_image');

		images = Array.prototype.map.call(images, function(image) {
			return [image, image.querySelector('img')];
		});

		function evt() {
			images.forEach(function(image) {
				var row = image[0].closest('.photonics_full'),
					rightTable = row.querySelector('.photonics_right'),
					afterImage = image[0].parentNode.querySelector('.photonics_after_image');
				
				var leftHeight = row.querySelector('.photonics_left').offsetHeight,
					rightHeight = rightTable.offsetHeight,
					rightMargin = parseInt(window.getComputedStyle(rightTable).getPropertyValue("margin-bottom")),
					afterHeight = (afterImage && afterImage.offsetHeight) || 0;

				image[0].style.height = (leftHeight - rightHeight - rightMargin - afterHeight) + 'px';
				image[1] && (image[1].style.maxHeight = (leftHeight - rightHeight - rightMargin - afterHeight) + 'px');
			});
		}

		var debounce = undefined;
		window.addEventListener('resize', function() {
			clearTimeout(debounce);
			debounce = setTimeout(evt, 200);
		}, false);

		var resizer = document.getElementById('photonics_entry_resize');
		resizer && Array.prototype.forEach.call(resizer.getElementsByTagName('iframe'), function(v) {
			(v.contentWindow || v).onresize = evt;
		});

		evt();
	})();

	(function formCategoryFields() {
		var select = document.getElementById('category');
		var fields = document.getElementsByClassName('photonics_category_field');
		
		function showCategoryFields(e) {
			var value = select.value;
			if (value === '__None') {
				Array.prototype.forEach.call(fields, function(v) {
					v.style.display = 'none';
				});
			} else {
				Array.prototype.forEach.call(fields, function(v) {
					v.style.display = v.dataset.id === value ? null : 'none';
				});
			}
		}

		if (select) {
			select.addEventListener('change', showCategoryFields);
			select.value !== '__None' && showCategoryFields();
		}
	})();

	(function formActualUse() {
		var select = document.getElementById('actual_use_user');
		select && select.addEventListener('change', function(e) {
			['actual_use_type', 'actual_use_start_date', 'actual_use_end_date'].forEach(function(v) {
				var element = document.getElementById(v);
				if (select.value === '__None') {
					element.disabled = true;
					element.value = element.nodeName === 'INPUT' ? null : '__None';
				} else {
					element.disabled = false;
				}
			});
		});
	})();

	(function formDeleteImage() {
		var deleteImage = document.querySelector('[data-modal-button-trigger="delete_image"]');
		deleteImage && deleteImage.addEventListener('delete_image', function(e) {
			var image = document.getElementById('photonics_image_col'),
				upload = document.getElementById('photonics_image_upload_col'),
				form = image.closest('form');

			image.parentNode.removeChild(image);

			['col-lg-7', 'col-md-7', 'col-sm-6', 'col-xs-6'].forEach(function(v) {
				upload.classList.remove(v);
			});

			['col-lg-12', 'col-md-12', 'col-sm-12', 'col-xs-12'].forEach(function(v) {
				upload.classList.add(v);
			});

			var deleteInput = createElement('input', {
				type: 'hidden',
				name: 'delete_image',
				value: 'true'
			});

			form.appendChild(deleteInput);
		});
	})();

	(function formDeleteFile() {
		var selectedFile = null;
		var deleteButton = document.getElementsByClassName('photonics_file_delete');

		Array.prototype.forEach.call(deleteButton, function(v) {
			v.addEventListener('click', function(e) {
				e && e.preventDefault() && e.stopPropagation();
				selectedFile = v.closest('.photonics_file').dataset.fileId;
			});
		});

		var deleteFile = document.querySelector('[data-modal-button-trigger="delete_file"]');
		deleteFile && deleteFile.addEventListener('delete_file', function(e) {
			var file = document.querySelector('.photonics_file[data-file-id="' + selectedFile + '"]'),
				form = file.closest('form');

			file.parentNode.removeChild(file);

			var deleteInput = createElement('input', {
				type: 'hidden',
				name: 'delete_file-' + selectedFile,
				value: 'true'
			});
			
			form.appendChild(deleteInput);
			selectedFile = null;
		});
	})();

})(window, document);
