;(function(window, document, undefined) {
	'use strict';

	(function csrfToken() {
		var csrfToken = document.querySelector('meta[name="csrf-token"]').content;
		window.csrfToken = csrfToken;

		var formInputs = document.querySelectorAll('.auto_csrf_token');

		Array.prototype.forEach.call(formInputs, function(input) {
			input.value = csrfToken;
		});
	})();

	(function actualRoute() {
		var path = window.location.pathname.split('/').filter(function(v) { return v; });
		
		if (['inventory', 'category'].every(function(v, i) { return path[i] === v; })) {
			var categories = document.querySelectorAll('#sidebar-menu .photonics_category');
			var category = Array.prototype.find.call(categories, function(v) {
				return v.dataset.category === path[2];
			});

			category && category.classList.add('active');

			var categorySelector = document.getElementById('category_selector');
			Array.prototype.every.call(categorySelector.options, function(o) {
				if (!o.value.startsWith(path[2] + '-')) {
					return true;
				}

				categorySelector.value = o.value;
				categorySelector.dataset.selected = o.value;
				return false;
			});
		} else if (['inventory', 'all'].every(function(v, i) { return path[i] === v; })) {
			var categories = document.querySelectorAll('#sidebar-menu .photonics_category');
			var category = Array.prototype.find.call(categories, function(v) {
				return v.dataset.category === "-1";
			});

			category && category.classList.add('active');
		}
	})();

	(function hashAction() {
		var hash = window.location.hash;
		var index = (hash && hash.indexOf('!')) || -1;
		var type = undefined;
		index != -1 && (hash = window.atob(hash.substr(index + 1)));

		if (hash) {
			index = hash.indexOf('_');
			if (index > 0 && index < hash.length - 1) {
				type = hash.substr(0, index);
				hash = hash.substr(index + 1);

				var actions = {
					url: function(h) {
						var element = document.querySelector('.photonics_entry .photonics_entry_back');
						if (element) {
							element.setAttribute('href', h + element.getAttribute('href'));
							element.style.display = '';
						}
					},

					entry: function(h) {
						var id = window.btoa('entry_' + h);
						var element = document.getElementById(id);

						if (element) {
							var scroll = element.closest('.row').offsetTop - document.querySelector('.top_nav').offsetHeight - 10;
							makeCollapser(element, element.querySelector('.collapse-link').querySelector('i'))('force');
							window.scrollTo(0, scroll);
						}
					}
				};

				Object.prototype.hasOwnProperty.call(actions, type) && actions[type](hash);
			}
		}
	})();

	(function notifications() {
		var closeButtons = document.querySelectorAll('.notification-close');

		Array.prototype.forEach.call(closeButtons, function(button) {
			button.addEventListener('click', function(e) {
				e && e.preventDefault() && e.stopPropagation();
				var wrapper = button.closest('.custom-notifications');
				wrapper && (wrapper.style.display = 'none');
			});
		});
	})();

	(function modals() {
		var modals = document.querySelectorAll('[data-toggle="modal"]');
		var nonTriggableModals = document.querySelectorAll('.modal[data-modal-ready="false"]');

		function showModal(modal, dialog, e) {
			var trigger = modal.dataset.modalTrigger;
			var prevent = true;
			if (trigger) {
				prevent = !modal.dispatchEvent(new CustomEvent(trigger, { detail: e }));
			}

			if (prevent) {
				e && e.preventDefault() && e.stopPropagation();
			}

			if (dialog.dataset.openable !== 'false') {
				e && e.preventDefault() && e.stopPropagation();	
				dialog.style.display = 'table';
				dialog.classList.add('modal-open');
			}
		}

		function createDialog(dialog) {
			function hideModal(e) {
				if (closable) {
					e && e.preventDefault() && e.stopPropagation();
					dialog.style.display = 'none';
					dialog.classList.remove('modal-open');
				}
			}

			if (dialog.dataset['modalReady'] !== 'true') {
				var dialogWrapper = dialog.querySelector('.modal-wrapper');
				var closable = dialog.dataset.closable !== 'false';
				var triggerButtons = dialog.querySelectorAll('button[data-modal-button-trigger]');
				var closeButtons = dialog.querySelectorAll('[data-dismiss="modal"]');

				dialog.addEventListener('click', function(e) {
					if (e && e.target === dialogWrapper) {
						hideModal(e);
					}
				});

				closable && Array.prototype.forEach.call(closeButtons, function(button) {
					button.addEventListener('click', hideModal);
				});

				Array.prototype.forEach.call(triggerButtons, function(button) {
					button.addEventListener('click', function(e) {
						e && e.preventDefault() && e.stopPropagation();
						var dismiss = button.dispatchEvent(new CustomEvent(button.dataset.modalButtonTrigger, { detail: e }));
						dismiss && hideModal(e);
					});
				});

				dialog.dataset['modalReady'] = 'true';
			}
		}

		function createModal(modal, trigger) {
			modal.dataset['modalReady'] = 'true';
			var dialog = document.querySelector(modal.dataset.target);

			modal.addEventListener(modal.dataset.modalEvent || 'click', function(e) {
				showModal(modal, dialog, e);
			});

			createDialog(dialog);

			trigger && showModal(modal, dialog);
		}

		Array.prototype.forEach.call(modals, function(v) { createModal(v); });
		Array.prototype.forEach.call(nonTriggableModals, createDialog);

		window.addEventListener('click', function(e) {
			var src = e && e.target;
			var data = src && src.dataset;

			if (src && data && data.toggle === 'modal' && data['modalReady'] !== 'true') {
				e.preventDefault() && e.stopPropagation();
				createModal(src, true);
			}
		});

		window.addEventListener('keyup', function(e) {
			if (e && e.keyCode === 27 && (!e.target || !e.target.classList.contains('form-control'))) {
				var dialogs = document.querySelectorAll('.modal-open');
				Array.prototype.forEach.call(dialogs, function(dialog) {
					if (dialog.dataset.closable !== 'false') {
						dialog.style.display = 'none';
						dialog.classList.remove('modal-open');
					}
				});
			}
		});
	})();

	(function modalResize() {
		function resizeModal(height) {
			return function(modal) {
				var content = modal.closest('.modal-content');
				var header = content.querySelector('.modal-header') && 56;
				var footer = content.querySelector('.modal-footer') && 65;
				modal.style.maxHeight = ((height * 0.90) - (header || 0) - (footer || 0)) + 'px';
			}
		}

		function resizeModals() {
			var modals = document.querySelectorAll('.modal-body');
			var height = window.innerHeight;
			var resize = resizeModal(height);
			modals.forEach(resize);
		}

		resizeModals();

		var debounce = undefined;
		window.addEventListener('resize', function() {
			clearTimeout(debounce);
			debounce = setTimeout(resizeModals, 200);
		}, false);
	})();

	(function dropdowns() {
		var dropdowns = document.querySelectorAll('.dropdown-toggle');

		Array.prototype.forEach.call(dropdowns, function(dropdown) {
			var li = dropdown.closest('.dropdown');
			var menu = li.querySelector('.dropdown-menu');

			function toggle(e) {
				e && e.preventDefault() && e.stopPropagation();
				li.classList.toggle('open');
			}

			dropdown.addEventListener('click', toggle, false);
			window.addEventListener('click', function(e) {
				e && li.classList.contains('open')
					&& e.target.closest('.dropdown') != li
					&& toggle();
			}, false);
		});
	})();

	(function trees() {
		var trees = document.querySelectorAll('.nav-tree > li');

		Array.prototype.forEach.call(trees, function(tree) {
			var collapseLink = tree.querySelector('.collapse-link'),
				collapseAnchor = tree.querySelector('.collapse-anchor');

			[collapseLink, collapseAnchor].forEach(function(v) {
				v && v.addEventListener(
					'click',
					makeCollapser(tree, collapseLink),
					false);
			});
		});
	})();

	(function sidebar() {
		var categories = document.querySelectorAll('#sidebar-menu .photonics_category');

		Array.prototype.forEach.call(categories, function(category) {
			var parent = category.closest('li'),
				collapser = category.querySelector('.collapse-link');

			if (collapser) {
				var evt = makeCollapser(parent, collapser);
				collapser.addEventListener('click', evt, false);
				parent.getElementsByClassName('active').length && evt();
			}
		});
	})();

	(function categorySelector() {
		var categorySelector = document.getElementById('category_selector');
		var selectedCategory = categorySelector.dataset.selected || '__None';

		categorySelector.addEventListener('change', function(e) {
			e && e.preventDefault() && e.stopPropagation();
			if (categorySelector.value != '__None') {
				var value = categorySelector.value,
					index = value.indexOf('-');

				value = value.substring(0, index) + '/' + value.substring(index + 1);
				window.location.href = '/inventory/category/' + value;
			} else {
				categorySelector.value = selectedCategory;
			}
		});
	})();

	(function search() {
		var form = document.getElementById('search'),
			input = form.querySelector('input');

		function evt(e) {
			e && e.preventDefault() && e.stopPropagation();
			var search = encodeURIComponent(input.value.trim());
			search.length && (window.location.href = '/inventory/search/query/' + search);
		}

		form.addEventListener('submit', evt, false);
	})();

	(function advancedSearch() {
		var categorySelect = document.getElementById('adv_search_category');
		var categoryFields = document.getElementsByClassName('adv_search_photonics_category_field');

		function showCategoryFields(e) {
			var value = categorySelect.value;
			Array.prototype.forEach.call(categoryFields, function(v) {
				v.style.display = value === '__None' || v.dataset.id !== value ? 'none' : null;
			});
		}

		function disableActualUseFields(e) {
			var element = document.getElementById('adv_search_actual_use_type');
			if (useSelect.value === '0') {
				element.disabled = true;
				element.value = '__None';
			} else {
				element.disabled = false;
			}
		}

		if (categorySelect) {
			categorySelect.addEventListener('change', showCategoryFields);
			categorySelect.value !== '__None' && showCategoryFields();
		}

		var useSelect = document.getElementById('adv_search_actual_use_user');
		if (useSelect) {
			useSelect.addEventListener('change', disableActualUseFields);
			disableActualUseFields();
		}
	})();

	(function clipboard() {
		var elements = document.getElementsByClassName('clipboard');
		Array.prototype.forEach.call(elements, function(v) {
			v.addEventListener('click', function(e) {
				e && e.preventDefault() && e.stopPropagation();
			});
		});

		new Clipboard('.clipboard');
	})();

	(function tooltips() {
		Opentip.styles.opentipped = {
			background: 'rgba(52, 73, 94, 0.88)',
			borderWidth: 0,
			borderRadius: 4,
			tipJoint: 'top',
			targetJoint: 'bottom',
			offset: [0, 2],
			target: true,
			stem: true,
			delay: 0.125,
			hideDelay: 0,
		};

		Opentip.defaultStyle = 'opentipped';
	})();

})(window, document);
