;(function(window, document, undefined) {
	'use strict';

	var hasAdvancedUpload = function() {
		var hop = Object.prototype.hasOwnProperty;
		if (!hop.call(window, 'CustomEvent')) {
			return false;
		}

		if (hop.call(window, 'FormData') && hop.call(window, 'FileReader')) {
			var div = document.createElement('div');
			return ('draggable' in div || ('ondragstart' in div && 'ondrop' in div));
		}

		return false;
	}();

	function compareFiles(file1, file2) {
		return ['name', 'size', 'type', 'lastModifieds'].every(function (v) {
			return file1[v] === file2[v];
		});
	}

	function appendFiles(target, files) {
		Array.prototype.slice.call(files).forEach(function(x) {
			var comparer = function(y) { return !compareFiles(x, y); };
			target.every(comparer) && target.push(x);
		});

		return target;
	}

	function showFiles(wrapper, label, files) {
		var text, tooltip;

		if (files.length === 0) {
			text = null;
			tooltip = null;

			wrapper.classList.remove('file-upload-files-hasfile');
		} else {
			text = files.length === 1
				? files[0].name
				: label.dataset.multipleCaption.replace('{count}', files.length);
			tooltip = files.reduce(function(x, v) { return x + '\n' + v.name; }, '').trim();

			wrapper.classList.add('file-upload-files-hasfile');
		}

		label.textContent = text;
		label.setAttribute('title', tooltip);
	}

	var uploads = document.getElementsByClassName('file-upload-wrapper');
	Array.prototype.forEach.call(uploads, function(wrapper) {
		wrapper.uploadjs = Object.create({ files: [] });
		
		var form = wrapper.closest('form');
		if (!form.dataset.uploadForm) {
			form.dataset.uploadForm = true;
		}

		var multiple = (wrapper.dataset['multiple'] === 'true' || undefined),
			input = wrapper.querySelector('.file-upload-input-file'),
			label = wrapper.querySelector('.file-upload-files-label'),
			cancel = wrapper.querySelector('.file-upload-cancel-label');

		var show = function() {
			return showFiles(wrapper, label, wrapper.uploadjs.files);
		};
		
		input.addEventListener('change', function(e) {
			e && e.preventDefault() && e.stopPropagation();

			if (multiple || e.target.files.length === 1) {
				wrapper.uploadjs.files = appendFiles(wrapper.uploadjs.files, e.target.files);
				show();
			}
		});

		cancel.addEventListener('click', function(e) {
			e && e.preventDefault() && e.stopPropagation();
			input.value = null;
			wrapper.uploadjs.files.length = 0;
			show();
		});

		if (hasAdvancedUpload) {
			wrapper.classList.add('has-advanced-upload');

			['drag', 'dragstart'].forEach(function(v) {
				wrapper.addEventListener(v, function(e) {
					e && e.preventDefault() && e.stopPropagation();
				});
			});

			['dragover', 'dragenter'].forEach(function(v) {
				wrapper.addEventListener(v, function(e) {
					e && e.preventDefault() && e.stopPropagation();
					wrapper.classList.add('is-dragover');
				});
			});

			['dragleave', 'dragend', 'drop'].forEach(function(v) {
				wrapper.addEventListener(v, function(e) {
					e && e.preventDefault() && e.stopPropagation();
					wrapper.classList.remove('is-dragover');
				});
			});

			wrapper.addEventListener('drop', function(e) {
				e && e.preventDefault() && e.stopPropagation();
				if (multiple || e.dataTransfer.files.length === 1) {
					wrapper.uploadjs.files = appendFiles(wrapper.uploadjs.files, e.dataTransfer.files);
					show();
				}
			});
		}
	});

	function formHasFiles(uploads) {
		return !Array.prototype.every.call(uploads, function(v) {
			return !v.uploadjs.files.length;
		});
	}

	function submitForm(evt, form, uploads) {
		if (formHasFiles(uploads)) {
			evt && evt.preventDefault() && evt.stopPropagation();
			if (form.dataset.toggle === 'modal') {
				document.querySelector(form.dataset.target).dataset.openable = 'true';
			}

			var data = new FormData(form);
			var uploads = form.getElementsByClassName('file-upload-wrapper');

			var output_data = new FormData();
			var upload_entries = [];

			Array.prototype.forEach.call(uploads, function(v) {
				var input = v.querySelector('input');
				upload_entries.push(input.name);

				v.uploadjs.files.forEach(function(f) {
					output_data.append(input.name, f);
				});
			});

			for (var iterator = data.entries(), entry = iterator.next(); !entry.done; entry = iterator.next()) {
				if (!(upload_entries.indexOf(entry.value[0]) + 1)) {
					output_data.append(entry.value[0], entry.value[1]);
				}
			}

			output_data.append('xhr', 'xhr');
			var request = new XMLHttpRequest();
			request.onreadystatechange = function() {
				if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
					console.log(request.responseText);
					if (request.responseText === '__Error') {
						window.location.reload();
					} else {
						window.location = request.responseText;
					}
				}
			};

			request.open(form.method, form.action.length ? form.location : window.location, true);
			request.send(output_data);
		} else if (form.dataset.toggle === 'modal') {
			document.querySelector(form.dataset.target).dataset.openable = 'false';
		}
	}

	var forms = document.querySelectorAll('form[data-upload-form="true"]');
	Array.prototype.forEach.call(forms, function(form) {
		var uploads = form.getElementsByClassName('file-upload-wrapper');
		form.addEventListener(form.dataset.submitUpload === 'true' ? 'submit-upload' : 'upload', function(e) {
			submitForm(e, form, uploads);
		});
	});

})(window, document);
