# -*- coding: utf-8 -*-

import base64
import email.message
import functools
import hashlib
import re
import smtplib
import unicodedata
import urllib.parse
import uuid

import config

def normalize(string, allowed_chars=''):
    string = unicodedata.normalize('NFKD', string)
    string = string.encode('ascii', 'ignore')
    string = string.decode('ascii')
    string = re.sub('[ ]+', '_', string)
    string = re.sub('[^' + allowed_chars + '0-9a-zA-Z_-]', '', string)
    string = re.sub('_{2,}', '_', string)
    return string.lower()


def base64_encode(string):
    string = string.encode('ascii', 'ignore')
    string = base64.b64encode(string)
    return string.decode('ascii')


def base64_decode(string):
    string = string.encode('ascii')
    string = base64.b64decode(string)
    return string.decode('ascii', 'ignore')


def urlforce(string):
    return '_' if not string else string


def urljoin(url, base_url):
    return urllib.parse.urljoin(base_url, url)


def reduce_form_errors(form):
    errors = form.errors
    if not errors:
        return []
        
    reduced_errors = functools.reduce(list.__add__, (x for x in form.errors.values()))
    return set(x for x in reduced_errors if x)


def mime_to_icon(mime):
    mimes = (
        ['text/plain', 'fa-file-text-o'],
        ['image/', 'fa-file-image-o'],
        ['audio/', 'fa-file-audio-o'],
        ['video/', 'fa-file-video-o'],
        ['application/pdf', 'fa-file-pdf-o'],
        ['x-compressed', 'fa-file-archive-o'])

    return next((x[1] for x in mimes if x[0] in mime), 'fa-file-o')


def get_file_extension(filename):
    if filename.rfind('.') == -1:
        return ''

    return filename[filename.rfind('.'):]


def get_md5_uuid():
    return hashlib.md5(str(uuid.uuid1()).encode('utf-8')).hexdigest()


def send_mail(to, subject, content, is_html=False):
    msg = email.message.Message()
    msg['Subject'] = subject
    msg['From'] = config.AppConfig.MAIL_FROM
    msg['To'] = to
    if is_html:
        msg.add_header('Content-Type','text/html')

    msg.set_payload(content)

    server = smtplib.SMTP(config.AppConfig.MAIL_SERVER, config.AppConfig.MAIL_PORT)
    server.starttls()
    server.login(config.AppConfig.MAIL_USERNAME, config.AppConfig.MAIL_PASSWORD)
    server.sendmail(msg['From'], [msg['To']], msg.as_string().encode('utf-8'))
