# -*- coding: utf-8 -*-

from flask import flash, request, redirect, url_for, send_file, abort, Response
from flask_login import current_user
from flask_babelex import gettext

import photon
import config
import utils
from shared import db, DatabaseUtils

import models.app
import models.account
import models.inventory
import forms.admin

from sqlalchemy.sql import exists
from sqlalchemy.sql.expression import and_
from sqlalchemy.exc import IntegrityError
from sqlalchemy.ext.serializer import loads, dumps

import codecs
import io
import json
import os
import re
import subprocess
import time

import zipfile
import zipstream

def delete_old_backups():
	command = "/usr/bin/find " + config.AppConfig.BACKUPS_DIRECTORY + " -type f -mmin +30 -delete"
	result = subprocess.Popen(command.split())
	result.communicate()
	result.wait()


def get_database_backup():
	t = str(int(time.time()))
	bak_path = config.AppConfig.BACKUPS_DIRECTORY + 'photonics_database_' + t + '.bak'
	tar_path = config.AppConfig.BACKUPS_DIRECTORY + 'photonics_database_' + t + '.tar'

	backup = DatabaseUtils().create_database_backup()
	with open(bak_path, 'w') as text_file:
		text_file.write(backup)

	command = "/usr/bin/tar -cf " + tar_path + " " + config.AppConfig.IMAGES_DIRECTORY
	result = subprocess.Popen(command.split())
	result.communicate()
	result.wait()

	zip_file = zipstream.ZipFile(mode='w', compression=zipstream.ZIP_DEFLATED)
	zip_file.write(tar_path, arcname='photonics_backup_database_blobs.bak')
	zip_file.write(bak_path, arcname='photonics_backup_database_data.bak')

	return zip_file


def get_files_backup():
	t = str(int(time.time()))
	tar_path = config.AppConfig.BACKUPS_DIRECTORY + 'photonics_files_' + t + '.tar'

	command = "/usr/bin/tar -cf " + tar_path + " " + config.AppConfig.FILES_DIRECTORY
	result = subprocess.Popen(command.split())
	result.communicate()
	result.wait()

	zip_file = zipstream.ZipFile(mode='w', compression=zipstream.ZIP_DEFLATED)
	zip_file.write(tar_path, arcname='photonics_backup_files_blobs.bak')

	return zip_file


@photon.route()
@photon.authenticated(admin=True)
def admin():
	delete_old_backups()
	return render_admin(None)


@photon.route(route='/admin/check_updates')
@photon.authenticated(admin=True)
def admin_check_updates():
	command = "/bin/yum check-update --quiet"
	result = subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	updates = str(result.stdout)
	updates = updates.split('\n')
	updates = len([x for x in updates if re.match('^a-zA-Z0-9', x)])
	return render_admin(updates)


def render_admin(updates):
	return photon.render_logged_template(
		'admin.html',
		users=models.account.User.query.order_by(models.account.User.first_name, models.account.User.last_name).all(),
		user_types=models.account.UserType.query.order_by(models.account.UserType.name).all(),
		notifications=create_notifications_form(),
		updates=updates)


def create_notifications_form():
	notifications = models.app.Notification.query.all()
	form = forms.admin.Notifications()

	for k, v in vars(form).items():
		if callable(v) and k.endswith('_message'):
			n = next((x for x in notifications if x.key == k), None)
			v.data = n.value if n else None

	return form


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_status_add():
	form = forms.admin.SimpleItemAdd()
	if form.validate():
		entry_status = models.inventory.EntryStatusType(form.name.data)
		db.session.add(entry_status)
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_status_type_add_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_status_edit():
	form = forms.admin.SimpleItemEdit()
	if form.validate():
		entry_status = models.inventory.EntryStatusType.query.get(form.id.data)
		entry_status.name = form.name.data
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_status_type_edit_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_status_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		entry_status = models.inventory.EntryStatusType.query.get(form.id.data)
		db.session.delete(entry_status)
		db.session.commit()
	else:
		flash(get('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_use_add():
	form = forms.admin.SimpleItemAdd()
	if form.validate():
		entry_use = models.inventory.EntryUseType(form.name.data)
		db.session.add(entry_use)
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_use_type_add_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_use_edit():
	form = forms.admin.SimpleItemEdit()
	if form.validate():
		entry_use = models.inventory.EntryUseType.query.get(form.id.data)
		entry_use.name = form.name.data
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_use_type_edit_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_use_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		entry_use = models.inventory.EntryUseType.query.get(form.id.data)
		db.session.delete(entry_use)
		db.session.commit()
	else:
		flash(gettext('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_documentation_add():
	form = forms.admin.SimpleItemAdd()
	if form.validate():
		entry_documentation = models.inventory.EntryDocumentationType(form.name.data)
		db.session.add(entry_documentation)
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_documentation_type_add_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_documentation_edit():
	form = forms.admin.SimpleItemEdit()
	if form.validate():
		entry_documentation = models.inventory.EntryDocumentationType.query.get(form.id.data)
		entry_documentation.name = form.name.data
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_documentation_type_edit_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_documentation_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		entry_documentation = models.inventory.EntryDocumentationType.query.get(form.id.data)
		db.session.delete(entry_documentation)
		db.session.commit()
	else:
		flash(get('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_field_add():
	form = forms.admin.SimpleItemAdd()
	if form.validate():
		entry_field = models.inventory.EntryField(form.name.data)
		db.session.add(entry_field)
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_field_add_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_field_edit():
	form = forms.admin.SimpleItemEdit()
	if form.validate():
		entry_field = models.inventory.EntryField.query.get(form.id.data)
		entry_field.name = form.name.data
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_entry_field_edit_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_entry_field_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		entry_field = models.inventory.EntryField.query.get(form.id.data)

		db.session.delete(entry_field)
		db.session.commit()
	else:
		flash(gettext('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_category_add():
	form = forms.admin.CategoryAdd()
	if form.validate():
		parent = models.inventory.Category.query.get(form.parent.data) if form.parent.data else None
		if not parent and form.parent.data:
			flash(gettext('unknown_error'), 'admin')
		else:
			category = models.inventory.Category(form.name.data, parent)
			db.session.add(category)
			try:
				db.session.commit()
			except:
				db.session.rollback()
				flash(gettext('integrity_category_add_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_category_edit():
	form = forms.admin.CategoryEdit()
	if form.validate():
		category = models.inventory.Category.query.get(form.id.data)

		category.name = form.name.data
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_category_edit_form'), 'admin')
			return redirect(url_for('admin'))

		for x in form.fields:
			if db.session.query(models.inventory.CategoryField.query.filter(and_(
				models.inventory.CategoryField.category_id == category.id,
				models.inventory.CategoryField.name == x.data,
				models.inventory.CategoryField.id != x.id[len('fields-'):])).exists()).scalar():
					flash(gettext('integrity_fields_category_edit_form'), 'admin')
					return redirect(url_for('admin'))

		field_list = []
		for x in form.new_fields:
			field_list.append(x.data)

		if db.session.query(models.inventory.CategoryField.query.filter(and_(
			models.inventory.CategoryField.category_id == category.id,
			models.inventory.CategoryField.name.in_(field_list))).exists()).scalar():
				flash(gettext('integrity_fields_category_edit_form'), 'admin')
				return redirect(url_for('admin'))

		category_fields = models.inventory.CategoryField.query.filter_by(category_id=category.id).all()
		for x in category_fields:
			field = next((y for y in form.fields if y.id == 'fields-' + str(x.id)), None)
			if not field:
				db.session.delete(x)
			else:
				x.name = field.data

		for x in form.new_fields:
			new_field = models.inventory.CategoryField()
			new_field.name = x.data
			new_field.category = category
			db.session.add(new_field)

		db.session.commit()

	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_category_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		category = models.inventory.Category.query.get(form.id.data)
		if not category:
			flash(gettext('unknown_error'), admin)
		else:
			children = category.get_flattened_descendants()
			for x in reversed(children):
				db.session.delete(x)

			db.session.delete(category)

			try:
				db.session.commit()
			except:
				db.session.rollback()
				flash(gettext('unknown_error'), 'admin')
	else:
		flash(gettext('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_user_type_add():
	form = forms.admin.SimpleItemAdd()
	if form.validate():
		user_type = models.account.UserType(form.name.data)
		db.session.add(user_type)
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_user_type_add_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_user_type_edit():
	form = forms.admin.SimpleItemEdit()
	if form.validate():
		user_type = models.account.UserType.query.get(form.id.data)
		user_type.name = form.name.data
		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('integrity_user_type_edit_form'), 'admin')
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_user_type_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		user_type = models.account.UserType.query.get(form.id.data)
		db.session.delete(user_type)
		db.session.commit()
	else:
		flash(gettext('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('GET', 'POST'))
@photon.authenticated(admin=True)
def admin_user_add():
	form = forms.admin.UserAdd()

	if request.method == 'POST':
		if form.validate():
			user = models.account.User.create_instance(form)
			db.session.add(user)

			try:
				db.session.commit()

				utils.send_mail(
					user.email,
					config.AppConfig.NEW_USER_SUBJECT[user.language],
					config.AppConfig.NEW_USER_CONTENT[user.language]
						.replace('{0}', form.user_name.data)
						.replace('{1}', form.password.data))

				return redirect(url_for('admin'))
			except:
				db.session.rollback()
				flash(gettext('integrity_user_add_form'), 'admin')
		else:
			errors = utils.reduce_form_errors(form)
			if not errors:
				flash(gettext('unknown_error'), 'admin')

			for error in errors:
				flash(error, 'admin')

	return photon.render_logged_template(
		'admin_user.html',
		title=gettext('add_user'),
		form=form,
		delete=False)


@photon.route(methods=('GET', 'POST'), query=('int:user_id',))
@photon.authenticated(admin=True)
def admin_user_edit(user_id):
	user = models.account.User.query.get(user_id)
	if not user:
		abort(404)
		return

	if request.method == 'POST':
		form = forms.admin.UserEdit()
		if form.validate():
			form.populate_obj(user)

			if form.password.data:
				user.password = models.account.User.hash_password(form.password.data)

			try:
				db.session.commit()
				return redirect(url_for('admin'))
			except:
				db.session.rollback()
				flash(gettext('integrity_user_edit_form'), 'admin')

		else:
			errors = utils.reduce_form_errors(form)
			if not errors:
				flash(gettext('unknown_error'), 'admin')

			for error in errors:
				flash(error, 'admin')
	else:
		form=create_edit_user_form(user)

	return photon.render_logged_template(
		'admin_user.html',
		title=gettext('edit_user'),
		form=form,
		user_id=user.id,
		delete=current_user.id != user.id)


def create_edit_user_form(user):
	form = forms.admin.UserEdit()

	if user:
		for k, v in vars(form).items():
			if callable(v) and k != 'csrf_token' and not k.endswith('password'):
				v.data = getattr(user, k)

	return form


@photon.route(methods=('POST',),)
@photon.authenticated(admin=True)
def admin_user_delete():
	form = forms.admin.SimpleItemDelete()
	if form.validate():
		if current_user.id != form.id.data:
			user = models.account.User.query.get(form.id.data)
			db.session.delete(user)

			try:
				db.session.commit()
				return redirect(url_for('admin'))
			except:
				db.session.rollback()
				flash('integrity_user_delete_form', 'admin')

		else:
			flash(get('integrity_user_delete_form'), 'admin')

	else:
		flash(get('unknown_error'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_notifications():
	form = forms.admin.Notifications()

	if form.validate():
		notifications = {key: value.data
			for key, value in vars(form).items()
			if callable(value) and key.endswith('_message')}

		for key, value in notifications.items():
			notification = models.app.Notification.query.get(key)
			if not notification:
				notification = models.app.Notification(key, value.strip())
				db.session.add(notification)
			else:
				notification.value = value.strip()

		db.session.commit()
	else:
		errors = utils.reduce_form_errors(form)
		if not errors:
			flash(gettext('unknown_error'), 'admin')

		for error in errors:
			flash(error, 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_backup_database_get():
	backup = get_database_backup()
	response = Response(backup, mimetype='application/zip')
	response.headers['Content-Disposition'] = 'attachment; filename=photonics_database.bak'
	return response


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_backup_database_set():
	form = forms.admin.BackupRestore()

	if not form.validate():
		flash(gettext('invalid_backup_form'), 'admin')
		return redirect(url_for('admin'))

	t = str(int(time.time()))
	filename = 'photonics_backup_database_' + t + '.bak'

	try:
		command = "/usr/bin/find " + config.AppConfig.BACKUPS_DIRECTORY + "photonics_backup_database_* -type f -delete"
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		form.backup_file.data.save(os.path.join(config.AppConfig.BACKUPS_DIRECTORY, filename))
		with zipfile.ZipFile(config.AppConfig.BACKUPS_DIRECTORY + filename, 'r') as zip_file:
			zip_file.extract('photonics_backup_database_blobs.bak', path=config.AppConfig.BACKUPS_DIRECTORY)
			zip_file.extract('photonics_backup_database_data.bak', path=config.AppConfig.BACKUPS_DIRECTORY)

		command = "/usr/bin/rm -rf " + config.AppConfig.IMAGES_DIRECTORY + "/*"
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		command = "/usr/bin/tar -xf " + config.AppConfig.BACKUPS_DIRECTORY + "photonics_backup_database_blobs.bak -C " + config.AppConfig.APP_DIRECTORY
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		command = "/usr/bin/find " + config.AppConfig.BACKUPS_DIRECTORY + "photonics_backup_database_* -type f -delete"
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		with codecs.open(config.AppConfig.BACKUPS_DIRECTORY + 'photonics_backup_database_data.bak', 'r', 'utf-8') as json_file:
			data = json_file.read()

		DatabaseUtils().recover_database_backup(data)
		flash(gettext('admin_backup_restore_finished'), 'admin_noerror')
	except:
		flash(gettext('error_restoring_database_backup'), 'admin')

	return redirect(url_for('admin'))


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_backup_files_get():
	backup = get_files_backup()
	response = Response(backup, mimetype='application/zip')
	response.headers['Content-Disposition'] = 'attachment; filename=photonics_files.bak'
	return response


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_backup_files_set():
	form = forms.admin.BackupRestore()
	
	if not form.validate():
		flash(gettext('invalid_backup_form'), 'admin')
		return redirect(url_for('admin'))

	t = str(int(time.time()))
	filename = 'photonics_backup_files_' + t + '.zip'

	try:
		command = "/usr/bin/find " + config.AppConfig.BACKUPS_DIRECTORY + "photonics_backup_files_* -type f -delete"
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		form.backup_file.data.save(os.path.join(config.AppConfig.BACKUPS_DIRECTORY, filename))
		with zipfile.ZipFile(config.AppConfig.BACKUPS_DIRECTORY + filename, 'r') as zip_file:
			zip_file.extract('photonics_backup_files_blobs.bak', path=config.AppConfig.BACKUPS_DIRECTORY)

		command = "/usr/bin/rm -rf " + config.AppConfig.FILES_DIRECTORY + "/*"
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		command = "/usr/bin/tar -xf " + config.AppConfig.BACKUPS_DIRECTORY + "photonics_backup_files_blobs.bak -C " + config.AppConfig.APP_DIRECTORY
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()

		command = "/usr/bin/find " + config.AppConfig.BACKUPS_DIRECTORY + "photonics_backup_files_* -type f -delete"
		result = subprocess.Popen(command.split())
		result.communicate()
		result.wait()
		flash(gettext('admin_backup_restore_finished'), 'admin_noerror')
	except:
		flash(gettext('error_restoring_files_backup'), 'admin')

	return redirect(url_for('admin'))


@photon.route(query=('path:authentication',))
def admin_backup_auto(authentication):
	delete_old_backups()

	if authentication and ':' in authentication:
		username = authentication[0:authentication.find(':')]
		password = authentication[authentication.find(':') + 1:]

		user = models.account.User.query.filter_by(user_name=username).first()
		if user == None or not user.check_password(password) or not user.can_login or not user.can_admin:
			abort(401)
		else:
			database_backup = get_database_backup()
			files_backup = get_files_backup()

			backup = zipstream.ZipFile(mode='w', compression=zipstream.ZIP_DEFLATED)
			backup.write_iter('photonics_database.bak', database_backup)
			backup.write_iter('photonics_files.bak', files_backup)

			response = Response(backup, mimetype='application/zip')
			response.headers['Content-Disposition'] = 'attachment; filename=photonics_auto_backup.zip'
			return response
	else:
		abort(401)


@photon.route(methods=('POST',))
@photon.authenticated(admin=True)
def admin_update():
	command = ["/var/www/photonics/update.sh"]
	result = subprocess.Popen(command)
	flash(gettext('updating_system'), 'updates')
	return redirect(url_for('admin'))
